==> swirc-1.9.tgz: swirc-1.9/src/init_once.c <==
/* WIN32 one-time initialization
   Copyright (C) 2012-2014 Markus Uhlin. All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   - Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.

   - Neither the name of the author nor the names of its contributors may be
     used to endorse or promote products derived from this software without
     specific prior written permission.

   THIS SOFTWARE IS PROVIDED THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE. */

#include "common.h"

#include <intrin.h> /* _InterlockedCompareExchange() */

#include "init_once.h"

/* A replacement for W32's InitOnceExecuteOnce() -- for implementations that
   lacks support for it.  UNIX has pthread_once() for this purpose... */

int
init_once(init_once_t *once_control, PTR_TO_INIT_ROUTINE init_routine)
{
    if (once_control == NULL || init_routine == NULL) {
	return EINVAL;
    } else if (*once_control == ONCE_INITIALIZER && _InterlockedCompareExchange(
		   once_control, ONCE_EXCHANGE_VALUE, ONCE_INITIALIZER) == ONCE_INITIALIZER) {
	init_routine();
    } else {
	;
    }
    return 0;
}

==> swirc-3.1.0.tgz: swirc-3.1.0/src/init_once.c <==
/* WIN32 one-time initialization
   Copyright (C) 2012-2014 Markus Uhlin. All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   - Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.

   - Neither the name of the author nor the names of its contributors may be
     used to endorse or promote products derived from this software without
     specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
   BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE. */

#include "common.h"

#include <intrin.h> /* _InterlockedCompareExchange() */

#include "init_once.h"

/**
 * A replacement for Windows InitOnceExecuteOnce() -- for
 * implementations that lacks support for it. UNIX has pthread_once()
 * for this purpose.
 */
int
init_once(init_once_t *once_control, PTR_TO_INIT_ROUTINE init_routine)
{
    if (once_control == NULL || init_routine == NULL) {
	return EINVAL;
    } else if (*once_control == ONCE_INITIALIZER &&
	_InterlockedCompareExchange(
	    once_control,
	    ONCE_EXCHANGE_VALUE,
	    ONCE_INITIALIZER) == ONCE_INITIALIZER) {
	init_routine();
    }
    return 0;
}
